@@ -42,10 +42,19 @@ const µBlock = (( ) => { // jshint ignore:line
         autoUpdateAssetFetchPeriod: 120,
         autoUpdateDelayAfterLaunch: 180,
         autoUpdatePeriod: 7,
+        benchmarkDatasetURL: 'unset',
         blockingProfiles: '11111/#F00 11011/#C0F 11001/#00F 00001',
         cacheStorageAPI: 'unset',
         cacheStorageCompression: true,
         cacheControlForFirefox1376932: 'no-cache, no-store, must-revalidate',
+        cnameIgnoreList: 'unset',
+        cnameIgnore1stParty: true,
+        cnameIgnoreExceptions: true,
+        cnameIgnoreRootDocument: true,
+        cnameMaxTTL: 120,
+        cnameReplayFullURL: false,
+        cnameUncloak: true,
+        cnameUncloakProxied: false,
         consoleLogLevel: 'unset',
         debugScriptlets: false,
         debugScriptletInjector: false,
@@ -57,10 +66,14 @@ const µBlock = (( ) => { // jshint ignore:line
         loggerPopupType: 'popup',
         manualUpdateAssetFetchPeriod: 500,
         popupFontSize: 'unset',
+        popupPanelDisabledSections: 0,
+        popupPanelLockedSections: 0,
         requestJournalProcessPeriod: 1000,
         selfieAfter: 3,
         strictBlockingBypassDuration: 120,
         suspendTabsUntilReady: 'unset',
+        uiPopupConfig: 'undocumented',
+        uiFlavor: 'unset',
         updateAssetBypassBrowserCache: false,
         userResourcesLocation: 'unset',
     };
@@ -81,6 +94,7 @@ const µBlock = (( ) => { // jshint ignore:line
             ignoreGenericCosmeticFilters: vAPI.webextFlavor.soup.has('mobile'),
             largeMediaSize: 50,
             parseAllABPHideFilters: true,
+            popupPanelSections: 0b111,
             prefetchingDisabled: true,
             requestLogMaxEntries: 1000,
             showIconBadge: true,
@@ -89,28 +103,7 @@ const µBlock = (( ) => { // jshint ignore:line
         },
 
         hiddenSettingsDefault: hiddenSettingsDefault,
-        hiddenSettings: (( ) => {
-            const out = Object.assign({}, hiddenSettingsDefault);
-            const json = vAPI.localStorage.getItem('immediateHiddenSettings');
-            if ( typeof json !== 'string' ) { return out; }
-            try {
-                const o = JSON.parse(json);
-                if ( o instanceof Object ) {
-                    for ( const k in o ) {
-                        if ( out.hasOwnProperty(k) ) { out[k] = o[k]; }
-                    }
-                    self.log.verbosity = out.consoleLogLevel;
-                    if ( typeof out.suspendTabsUntilReady === 'boolean' ) {
-                        out.suspendTabsUntilReady = out.suspendTabsUntilReady
-                            ? 'yes'
-                            : 'unset';
-                    }
-                }
-            }
-            catch(ex) {
-            }
-            return out;
-        })(),
+        hiddenSettings: Object.assign({}, hiddenSettingsDefault),
 
         // Features detection.
         privacySettingsSupported: vAPI.browserSettings instanceof Object,
@@ -126,6 +119,7 @@ const µBlock = (( ) => { // jshint ignore:line
             'about-scheme',
             'chrome-extension-scheme',
             'chrome-scheme',
+            'edge-scheme',
             'moz-extension-scheme',
             'opera-scheme',
             'vivaldi-scheme',
@@ -141,8 +135,8 @@ const µBlock = (( ) => { // jshint ignore:line
 
         // Read-only
         systemSettings: {
-            compiledMagic: 23,  // Increase when compiled format changes
-            selfieMagic: 25,    // Increase when selfie format changes
+            compiledMagic: 26,  // Increase when compiled format changes
+            selfieMagic: 26,    // Increase when selfie format changes
         },
 
         // https://github.com/uBlockOrigin/uBlock-issues/issues/759#issuecomment-546654501
@@ -175,6 +169,10 @@ const µBlock = (( ) => { // jshint ignore:line
         selectedFilterLists: [],
         availableFilterLists: {},
 
+        // https://github.com/uBlockOrigin/uBlock-issues/issues/974
+        //   This can be used to defer filtering decision-making.
+        readyToFilter: false,
+
         pageStores: new Map(),
         pageStoresToken: 0,
 
