@@ -59,6 +59,8 @@ const redirectableResources = new Map([
     [ 'amazon_ads.js', {
         alias: 'amazon-adsystem.com/aax2/amzn_ads.js',
     } ],
+    [ 'amazon_apstag.js', {
+    } ],
     [ 'ampproject_v0.js', {
         alias: 'ampproject.org/v0.js',
     } ],
@@ -154,6 +156,7 @@ const redirectableResources = new Map([
         alias: 'scorecardresearch.com/beacon.js',
     } ],
     [ 'window.open-defuser.js', {
+        alias: 'nowoif.js',
         data: 'text',
     } ],
 ]);
@@ -446,6 +449,7 @@ RedirectEngine.prototype.compileRuleFromStaticFilter = function(line) {
                 .replace(/\*/g, '[\\w.%-]*')
                 .replace(/\./g, '\\.') +
             path
+                .replace(/\|$/, '$')
                 .replace(/[.+?{}()|[\]\/\\]/g, '\\$&')
                 .replace(/\^/g, '[^\\w.%-]')
                 .replace(/\*/g, '.*?');
