@@ -112,7 +112,6 @@
                     : 'unset';
         }
     }
-    self.log.verbosity = this.hiddenSettings.consoleLogLevel;
     this.fireDOMEvent('hiddenSettingsChanged');
 };
 
@@ -131,10 +130,23 @@
         }
     }
     vAPI.storage.set(bin);
-    this.saveImmediateHiddenSettings();
-    self.log.verbosity = this.hiddenSettings.consoleLogLevel;
 };
 
+self.addEventListener('hiddenSettingsChanged', ( ) => {
+    const µbhs = µBlock.hiddenSettings;
+    self.log.verbosity = µbhs.consoleLogLevel;
+    vAPI.net.setOptions({
+        cnameIgnoreList: µbhs.cnameIgnoreList,
+        cnameIgnore1stParty: µbhs.cnameIgnore1stParty,
+        cnameIgnoreExceptions: µbhs.cnameIgnoreExceptions,
+        cnameIgnoreRootDocument: µbhs.cnameIgnoreRootDocument,
+        cnameMaxTTL: µbhs.cnameMaxTTL,
+        cnameReplayFullURL: µbhs.cnameReplayFullURL,
+        cnameUncloak: µbhs.cnameUncloak,
+        cnameUncloakProxied: µbhs.cnameUncloakProxied,
+    });
+});
+
 /******************************************************************************/
 
 µBlock.hiddenSettingsFromString = function(raw) {
@@ -156,7 +168,7 @@
             }
             break;
         case 'string':
-            out[name] = value;
+            out[name] = value.trim();
             break;
         case 'number':
             out[name] = parseInt(value, 10);
@@ -181,33 +193,6 @@
 
 /******************************************************************************/
 
-// These settings must be available immediately on startup, without delay
-// through the vAPI.localStorage. Add/remove settings as needed.
-
-µBlock.saveImmediateHiddenSettings = function() {
-    const props = [
-        'consoleLogLevel',
-        'disableWebAssembly',
-        'suspendTabsUntilReady',
-    ];
-    const toSave = {};
-    for ( const prop of props ) {
-        if ( this.hiddenSettings[prop] !== this.hiddenSettingsDefault[prop] ) {
-            toSave[prop] = this.hiddenSettings[prop];
-        }
-    }
-    if ( Object.keys(toSave).length !== 0 ) {
-        vAPI.localStorage.setItem(
-            'immediateHiddenSettings',
-            JSON.stringify(toSave)
-        );
-    } else {
-        vAPI.localStorage.removeItem('immediateHiddenSettings');
-    }
-};
-
-/******************************************************************************/
-
 µBlock.savePermanentFirewallRules = function() {
     vAPI.storage.set({
         dynamicFilteringString: this.permanentFirewall.toString()
@@ -620,8 +605,7 @@
 µBlock.loadFilterLists = (( ) => {
     const loadedListKeys = [];
     let loadingPromise;
-
-    const t0 = Date.now();
+    let t0 = 0;
 
     const onDone = function() {
         log.info(`loadFilterLists() took ${Date.now()-t0} ms`);
@@ -698,6 +682,7 @@
 
     return function() {
         if ( loadingPromise instanceof Promise === false ) {
+            t0 = Date.now();
             loadedListKeys.length = 0;
             loadingPromise = Promise.all([
                 this.getAvailableLists().then(lists =>
@@ -817,16 +802,18 @@
     const reIsWhitespaceChar = /\s/;
     const reMaybeLocalIp = /^[\d:f]/;
     const reIsLocalhostRedirect = /\s+(?:0\.0\.0\.0|broadcasthost|localhost|local|ip6-\w+)\b/;
-    const reLocalIp = /^(?:0\.0\.0\.0|127\.0\.0\.1|::1|fe80::1%lo0)/;
+    const reLocalIp = /^(?:(0\.0\.0\.)?0|127\.0\.0\.1|::1?|fe80::1%lo0)\s+/;
     const lineIter = new this.LineIterator(this.processDirectives(rawText));
 
     while ( lineIter.eot() === false ) {
-        // rhill 2014-04-18: The trim is important here, as without it there
-        // could be a lingering `\r` which would cause problems in the
-        // following parsing code.
         let line = lineIter.next().trim();
         if ( line.length === 0 ) { continue; }
 
+        while ( line.endsWith(' \\') ) {
+            if ( lineIter.peek(4) !== '    ' ) { break; }
+            line = line.slice(0, -2).trim() + lineIter.next().trim();
+        }
+
         // Strip comments
         const c = line.charAt(0);
         if ( c === '!' || c === '[' ) { continue; }
@@ -946,6 +933,7 @@
     [ 'env_chromium', 'chromium' ],
     [ 'env_edge', 'edge' ],
     [ 'env_firefox', 'firefox' ],
+    [ 'env_legacy', 'legacy' ],
     [ 'env_mobile', 'mobile' ],
     [ 'env_safari', 'safari' ],
     [ 'cap_html_filtering', 'html_filtering' ],
@@ -999,7 +987,7 @@
 /******************************************************************************/
 
 µBlock.loadPublicSuffixList = async function() {
-    if ( this.hiddenSettings.disableWebAssembly === false ) {
+    if ( this.hiddenSettings.disableWebAssembly !== true ) {
         publicSuffixList.enableWASM();
     }
 
@@ -1009,8 +997,7 @@
             return;
         }
     } catch (ex) {
-        console.error(ex);
-        return;
+        log.info(ex);
     }
 
     const result = await this.assets.get(this.pslAssetKey);
@@ -1157,6 +1144,8 @@
         const json = await vAPI.adminStorage.getItem('adminSettings');
         if ( typeof json === 'string' && json !== '' ) {
             data = JSON.parse(json);
+        } else if ( json instanceof Object ) {
+            data = json;
         }
     } catch (ex) {
         console.error(ex);
@@ -1260,7 +1249,7 @@
 
 /******************************************************************************/
 
-µBlock.scheduleAssetUpdater = (function() {
+µBlock.scheduleAssetUpdater = (( ) => {
     let timer, next = 0;
 
     return function(updateDelay) {
@@ -1284,7 +1273,8 @@
             next = 0;
             this.assets.updateStart({
                 delay: this.hiddenSettings.autoUpdateAssetFetchPeriod * 1000 ||
-                       120000
+                       120000,
+                auto: true,
             });
         }, updateDelay);
     };
