@@ -96,9 +96,16 @@ const onBeforeRequest = function(details) {
 
     // Not blocked
     if ( result !== 1 ) {
-        if ( details.parentFrameId !== -1 && details.type === 'sub_frame' ) {
+        if (
+            details.parentFrameId !== -1 &&
+            details.type === 'sub_frame' &&
+            details.aliasURL === undefined
+        ) {
             pageStore.setFrame(details.frameId, details.url);
         }
+        if ( result === 2 ) {
+            return { cancel: false };
+        }
         return;
     }
 
@@ -447,11 +454,7 @@ const onHeadersReceived = function(details) {
     const responseHeaders = details.responseHeaders;
 
     if ( requestType === 'image' || requestType === 'media' ) {
-        return foilLargeMediaElement(
-            fctxt,
-            pageStore,
-            responseHeaders
-        );
+        return foilLargeMediaElement(fctxt, pageStore, responseHeaders);
     }
 
     if ( isDoc === false ) { return; }
@@ -1032,15 +1035,7 @@ const strictBlockBypasser = {
 return {
     start: (( ) => {
         vAPI.net = new vAPI.Net();
-
-        if (
-            vAPI.net.canSuspend() &&
-            µBlock.hiddenSettings.suspendTabsUntilReady !== 'no' ||
-            vAPI.net.canSuspend() !== true &&
-            µBlock.hiddenSettings.suspendTabsUntilReady === 'yes'
-        ) {
-            vAPI.net.suspend(true);
-        }
+        vAPI.net.suspend();
 
         return function() {
             vAPI.net.setSuspendableListener(onBeforeRequest);
