@@ -181,12 +181,14 @@ vAPI.browserSettings = (( ) => {
                 // https://github.com/gorhill/uBlock/issues/3009
                 //   Firefox currently works differently, use
                 //   `default_public_interface_only` for now.
-                bpn.webRTCIPHandlingPolicy.set({
-                    value: vAPI.webextFlavor.soup.has('chromium')
-                        ? 'disable_non_proxied_udp'
-                        : 'default_public_interface_only',
-                    scope: 'regular',
-                });
+                // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/API/privacy/network#Browser_compatibility
+                //   Firefox 70+ supports `disable_non_proxied_udp`
+                const value =
+                    vAPI.webextFlavor.soup.has('firefox') &&
+                    vAPI.webextFlavor.major < 70
+                        ? 'default_public_interface_only'
+                        : 'disable_non_proxied_udp';
+                bpn.webRTCIPHandlingPolicy.set({ value, scope: 'regular' });
             }
         },
 
@@ -824,7 +826,14 @@ vAPI.setIcon = (( ) => {
             }
         }
 
-        if ( browserAction.setTitle !== undefined ) {
+        // Insert the badge text in the title if:
+        // - the platform does not support browserAction.setIcon(); OR
+        // - the rendering of the badge is disabled
+        if (
+            browserAction.setTitle !== undefined && (
+                browserAction.setIcon === undefined || (parts & 0b1000) !== 0
+            )
+        ) {
             browserAction.setTitle({
                 tabId: tab.id,
                 title: titleTemplate.replace(
@@ -934,6 +943,7 @@ vAPI.messaging = {
     onFrameworkMessage: function(request, port, callback) {
         const sender = port && port.sender;
         if ( !sender ) { return; }
+        const fromDetails = this.ports.get(port.name) || {};
         const tabId = sender.tab && sender.tab.id || undefined;
         const msg = request.msg;
         switch ( msg.what ) {
@@ -978,6 +988,14 @@ vAPI.messaging = {
                 callback();
             });
             break;
+        case 'localStorage': {
+            if ( fromDetails.privileged !== true ) { break; }
+            const args = msg.args || [];
+            vAPI.localStorage[msg.fn](...args).then(result => {
+                callback(result);
+            });
+            break;
+        }
         case 'userCSS':
             if ( tabId === undefined ) { break; }
             const details = {
@@ -1164,16 +1182,17 @@ vAPI.Net = class {
         browser.webRequest.onBeforeRequest.addListener(
             details => {
                 this.normalizeDetails(details);
-                if ( this.suspendDepth === 0 || details.tabId < 0 ) {
-                    if ( this.suspendableListener === undefined ) { return; }
-                    return this.suspendableListener(details);
+                if ( this.suspendDepth !== 0 && details.tabId >= 0 ) {
+                    return this.suspendOneRequest(details);
                 }
-                return this.suspendOneRequest(details);
+                return this.onBeforeSuspendableRequest(details);
             },
             this.denormalizeFilters({ urls: [ 'http://*/*', 'https://*/*' ] }),
             [ 'blocking' ]
         );
     }
+    setOptions(/* options */) {
+    }
     normalizeDetails(/* details */) {
     }
     denormalizeFilters(filters) {
@@ -1199,6 +1218,8 @@ vAPI.Net = class {
     denormalizeTypes(types) {
         return types;
     }
+    canonicalNameFromHostname(/* hn */) {
+    }
     addListener(which, clientListener, filters, options) {
         const actualFilters = this.denormalizeFilters(filters);
         const actualListener = this.makeNewListenerProxy(clientListener);
@@ -1208,6 +1229,10 @@ vAPI.Net = class {
             options
         );
     }
+    onBeforeSuspendableRequest(details) {
+        if ( this.suspendableListener === undefined ) { return; }
+        return this.suspendableListener(details);
+    }
     setSuspendableListener(listener) {
         this.suspendableListener = listener;
     }
@@ -1242,11 +1267,55 @@ vAPI.Net = class {
             this.suspendDepth -= 1;
         }
         if ( this.suspendDepth !== 0 ) { return; }
-        this.unsuspendAllRequests(this.suspendableListener);
+        this.unsuspendAllRequests();
     }
     canSuspend() {
         return false;
     }
+    async benchmark() {
+        if ( typeof µBlock !== 'object' ) { return; }
+        const requests = await µBlock.loadBenchmarkDataset();
+        if ( Array.isArray(requests) === false || requests.length === 0 ) {
+            console.info('No requests found to benchmark');
+            return;
+        }
+        const mappedTypes = new Map([
+            [ 'document', 'main_frame' ],
+            [ 'subdocument', 'sub_frame' ],
+        ]);
+        console.info('vAPI.net.onBeforeSuspendableRequest()...');
+        const t0 = self.performance.now();
+        const promises = [];
+        const details = {
+            documentUrl: '',
+            tabId: -1,
+            parentFrameId: -1,
+            frameId: 0,
+            type: '',
+            url: '',
+        };
+        for ( const request of requests ) {
+            details.documentUrl = request.frameUrl;
+            details.tabId = -1;
+            details.parentFrameId = -1;
+            details.frameId = 0;
+            details.type = mappedTypes.get(request.cpt) || request.cpt;
+            details.url = request.url;
+            if ( details.type === 'main_frame' ) { continue; }
+            promises.push(this.onBeforeSuspendableRequest(details));
+        }
+        return Promise.all(promises).then(results => {
+            let blockCount = 0;
+            for ( const r of results ) {
+                if ( r !== undefined ) { blockCount += 1; }
+            }
+            const t1 = self.performance.now();
+            const dur = t1 - t0;
+            console.info(`Evaluated ${requests.length} requests in ${dur.toFixed(0)} ms`);
+            console.info(`\tBlocked ${blockCount} requests`);
+            console.info(`\tAverage: ${(dur / requests.length).toFixed(3)} ms per request`);
+        });
+    }
 };
 
 /******************************************************************************/
@@ -1341,6 +1410,78 @@ vAPI.adminStorage = (( ) => {
 /******************************************************************************/
 /******************************************************************************/
 
+// A localStorage-like object which should be accessible from the
+// background page or auxiliary pages.
+//
+// https://github.com/uBlockOrigin/uBlock-issues/issues/899
+//   Convert into asynchronous access API.
+//
+// Note: vAPI.localStorage should already be defined with the client-side
+//       implementation at this point, but we override with the
+//       background-side implementation.
+vAPI.localStorage = {
+    start: async function() {
+        if ( this.cache instanceof Promise ) { return this.cache; }
+        if ( this.cache instanceof Object ) { return this.cache; }
+        this.cache = webext.storage.local.get('localStorage').then(bin => {
+            this.cache = undefined;
+            try {
+                if (
+                    bin instanceof Object === false ||
+                    bin.localStorage instanceof Object === false
+                ) {
+                    this.cache = {};
+                    const ls = self.localStorage;
+                    for ( let i = 0; i < ls.length; i++ ) {
+                        const key = ls.key(i);
+                        this.cache[key] = ls.getItem(key);
+                    }
+                    webext.storage.local.set({ localStorage: this.cache });
+                } else {
+                    this.cache = bin.localStorage;
+                }
+            } catch(ex) {
+            }
+            if ( this.cache instanceof Object === false ) {
+                this.cache = {};
+            }
+        });
+        return this.cache;
+    },
+    clear: function() {
+        this.cache = {};
+        return webext.storage.local.set({ localStorage: this.cache });
+    },
+    getItem: function(key) {
+        if ( this.cache instanceof Object === false ) {
+            console.info(`localStorage.getItem('${key}') not ready`);
+            return null;
+        }
+        const value = this.cache[key];
+        return value !== undefined ? value : null;
+    },
+    getItemAsync: async function(key) {
+        await this.start();
+        const value = this.cache[key];
+        return value !== undefined ? value : null;
+    },
+    removeItem: async function(key) {
+        this.setItem(key);
+    },
+    setItem: async function(key, value = undefined) {
+        await this.start();
+        if ( value === this.cache[key] ) { return; }
+        this.cache[key] = value;
+        return webext.storage.local.set({ localStorage: this.cache });
+    },
+    cache: undefined,
+};
+
+vAPI.localStorage.start();
+
+/******************************************************************************/
+/******************************************************************************/
+
 // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/API/storage/sync
 
 vAPI.cloud = (( ) => {
@@ -1385,9 +1526,13 @@ vAPI.cloud = (( ) => {
 
     const options = {
         defaultDeviceName: window.navigator.platform,
-        deviceName: vAPI.localStorage.getItem('deviceName') || ''
+        deviceName: undefined,
     };
 
+    vAPI.localStorage.getItemAsync('deviceName').then(value => {
+        options.deviceName = value;
+    });
+
     // This is used to find out a rough count of how many chunks exists:
     // We "poll" at specific index in order to get a rough idea of how
     // large is the stored string.
@@ -1433,7 +1578,6 @@ vAPI.cloud = (( ) => {
     };
 
     const push = async function(dataKey, data) {
-
         let bin = {
             'source': options.deviceName || options.defaultDeviceName,
             'tstamp': Date.now(),
