@@ -116,6 +116,59 @@ vAPI.webextFlavor = {
 
 /******************************************************************************/
 
+{
+    const punycode = self.punycode;
+    const reCommonHostnameFromURL  = /^https?:\/\/([0-9a-z_][0-9a-z._-]*[0-9a-z])\//;
+    const reAuthorityFromURI       = /^(?:[^:\/?#]+:)?(\/\/[^\/?#]+)/;
+    const reHostFromNakedAuthority = /^[0-9a-z._-]+[0-9a-z]$/i;
+    const reHostFromAuthority      = /^(?:[^@]*@)?([^:]+)(?::\d*)?$/;
+    const reIPv6FromAuthority      = /^(?:[^@]*@)?(\[[0-9a-f:]+\])(?::\d*)?$/i;
+    const reMustNormalizeHostname  = /[^0-9a-z._-]/;
+
+    vAPI.hostnameFromURI = function(uri) {
+        let matches = reCommonHostnameFromURL.exec(uri);
+        if ( matches !== null ) { return matches[1]; }
+        matches = reAuthorityFromURI.exec(uri);
+        if ( matches === null ) { return ''; }
+        const authority = matches[1].slice(2);
+        if ( reHostFromNakedAuthority.test(authority) ) {
+            return authority.toLowerCase();
+        }
+        matches = reHostFromAuthority.exec(authority);
+        if ( matches === null ) {
+            matches = reIPv6FromAuthority.exec(authority);
+            if ( matches === null ) { return ''; }
+        }
+        let hostname = matches[1];
+        while ( hostname.endsWith('.') ) {
+            hostname = hostname.slice(0, -1);
+        }
+        if ( reMustNormalizeHostname.test(hostname) ) {
+            hostname = punycode.toASCII(hostname.toLowerCase());
+        }
+        return hostname;
+    };
+
+    const reHostnameFromNetworkURL =
+        /^(?:http|ws|ftp)s?:\/\/([0-9a-z_][0-9a-z._-]*[0-9a-z])(?::\d+)?\//;
+
+    vAPI.hostnameFromNetworkURL = function(url) {
+        const matches = reHostnameFromNetworkURL.exec(url);
+        return matches !== null ? matches[1] : '';
+    };
+
+    const psl = self.publicSuffixList;
+    const reIPAddressNaive = /^\d+\.\d+\.\d+\.\d+$|^\[[\da-zA-Z:]+\]$/;
+
+    vAPI.domainFromHostname = function(hostname) {
+        return reIPAddressNaive.test(hostname)
+            ? hostname
+            : psl.getDomain(hostname);
+    };
+}
+
+/******************************************************************************/
+
 vAPI.download = function(details) {
     if ( !details.url ) { return; }
     const a = document.createElement('a');
@@ -163,41 +216,38 @@ vAPI.closePopup = function() {
 
 // A localStorage-like object which should be accessible from the
 // background page or auxiliary pages.
-// This storage is optional, but it is nice to have, for a more polished user
-// experience.
-
-// https://github.com/gorhill/uBlock/issues/2824
-//   Use a dummy localStorage if for some reasons it's not available.
-
-// https://github.com/gorhill/uMatrix/issues/840
-//   Always use a wrapper to seamlessly handle exceptions
+//
+// https://github.com/uBlockOrigin/uBlock-issues/issues/899
+//   Convert into asynchronous access API.
 
 vAPI.localStorage = {
     clear: function() {
-        try {
-            window.localStorage.clear();
-        } catch(ex) {
-        }
+        vAPI.messaging.send('vapi', {
+            what: 'localStorage',
+            fn: 'clear',
+        });
     },
-    getItem: function(key) {
-        try {
-            return window.localStorage.getItem(key);
-        } catch(ex) {
-        }
-        return null;
+    getItemAsync: function(key) {
+        return vAPI.messaging.send('vapi', {
+            what: 'localStorage',
+            fn: 'getItemAsync',
+            args: [ key ],
+        });
     },
     removeItem: function(key) {
-        try {
-            window.localStorage.removeItem(key);
-        } catch(ex) {
-        }
+        return vAPI.messaging.send('vapi', {
+            what: 'localStorage',
+            fn: 'removeItem',
+            args: [ key ],
+        });
+    },
+    setItem: function(key, value = undefined) {
+        return vAPI.messaging.send('vapi', {
+            what: 'localStorage',
+            fn: 'setItem',
+            args: [ key, value ]
+        });
     },
-    setItem: function(key, value) {
-        try {
-            window.localStorage.setItem(key, value);
-        } catch(ex) {
-        }
-    }
 };
 
 
